import React, {Component} from 'react'
import PropTypes from 'prop-types'
import md5 from 'md5'
import { is, fromJS } from 'immutable'
import { Table, Typography, Col, Switch, message } from 'antd'

import asyncComponent from '@/utils/asyncComponent'
import { getMark } from '@/utils/utils.js'
import MKEmitter from '@/utils/events.js'
import Encrypts from '@/components/encrypts'
import './index.scss'

const { Paragraph } = Typography
const Video = asyncComponent(() => import('@/components/video'))
const MkIcon = asyncComponent(() => import('@/components/mk-icon'))
const MkPicture = asyncComponent(() => import('@/components/mkPicture'))
const CardCellComponent = asyncComponent(() => import('@/tabviews/custom/components/card/cardcellList'))
const PicRadio = {
  '4:3': '75%', '3:2': '66.67%', '16:9': '56.25%', '2:1': '50%', '3:1': '33.33%', '4:1': '25%',
  '5:1': '20%', '6:1': '16.67%', '7:1': '14.29%', '8:1': '12.5%', '9:1': '11.11%',
  '10:1': '10%', '3:4': '133.33%', '2:3': '150%', '9:16': '177.78%'
}
class BodyRow extends React.Component {
  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props.data), fromJS(nextProps.data)) || this.props.className !== nextProps.className
  }

  render() {
    let { lineMarks, onDoubleClick, data, className, ...resProps } = this.props
    let style = {}

    if (data && data.$disabled) {
      className = className + ' mk-disabled '
    }

    if (lineMarks && lineMarks.length > 0) {
      lineMarks.some(mark => {
        let originVal = data[mark.field[0]] + ''
        let contrastVal = ''
        let result = false

        if (mark.field[1] === 'static') {
          contrastVal = mark.contrastValue + ''
        } else {
          contrastVal = data[mark.field[2]] + ''
        }

        if (mark.match === '=') {
          result = originVal === contrastVal
        } else if (mark.match === '!=') {
          result = originVal !== contrastVal
        } else if (mark.match === 'like') {
          result = originVal.indexOf(contrastVal) > -1
        } else if (mark.match === '>') {
          try {
            originVal = parseFloat(originVal)
            contrastVal = parseFloat(contrastVal)
          } catch (e) {
            originVal = NaN
          }
  
          if (!isNaN(originVal) && !isNaN(contrastVal) && originVal > contrastVal) {
            result = true
          }
        } else if (mark.match === '<') {
          try {
            originVal = parseFloat(originVal)
            contrastVal = parseFloat(contrastVal)
          } catch (e) {
            originVal = NaN
          }
  
          if (!isNaN(originVal) && !isNaN(contrastVal) && originVal < contrastVal) {
            result = true
          }
        }

        if (result) {
          if (mark.signType[0] === 'font') {
            style.color = mark.color
          } else if (mark.signType[0] === 'background') {
            style.background = mark.color
            if (mark.fontColor) {
              style.color = mark.fontColor
            }
            className = className + ' background'
          } else if (mark.signType[0] === 'underline') {
            style.textDecoration = 'underline'
            style.color = mark.color
          } else if (mark.signType[0] === 'line-through') {
            style.textDecoration = 'line-through'
            style.color = mark.color
          }
        }

        return result
      })
    }

    return <tr {...resProps} onDoubleClick={onDoubleClick} className={className} style={style}/>
  }
}

class BodyCell extends React.Component {
  shouldComponentUpdate (nextProps, nextState) {
    return !nextProps.record || !is(fromJS(this.props.record), fromJS(nextProps.record))
  }

  render() {
    let { col, config, record, className, style, triggerLink, ...resProps } = this.props

    if (!col) return (<td {...resProps} className={className} style={style}/>)
    
    if (col.type === 'text') {
      let content = ''
      if (record[col.field] !== undefined) {
        content = `${record[col.field]}`
      }

      if (content !== '') {
        if (col.textFormat === 'YYYY-MM-DD' && /^[1-9]\d{3}(-|\/)(0[1-9]|1[0-2])(-|\/)(0[1-9]|[1-2][0-9]|3[0-1])/.test(content)) {
          content = `${content.substr(0, 4)}-${content.substr(5, 2)}-${content.substr(8, 2)}`
        } else if (col.textFormat === 'YYYY-MM-DD HH:mm:ss' && /^[1-9]\d{3}(-|\/)(0[1-9]|1[0-2])(-|\/)(0[1-9]|[1-2][0-9]|3[0-1]).([0-1][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9]/.test(content)) {
          content = `${content.substr(0, 4)}-${content.substr(5, 2)}-${content.substr(8, 2)} ${content.substr(11, 2)}:${content.substr(14, 2)}:${content.substr(17, 2)}`
        } else if (col.textFormat === 'encryption') {
          content = <span>{col.prefix || ''}<Encrypts value={content} />{col.postfix || ''}</span>
        }

        if (col.textFormat !== 'encryption') {
          content = (col.prefix || '') + content + (col.postfix || '')
        }
      }

      if (col.marks) {
        style = style ? {...style} : {}

        let mark = getMark(col.marks, record, style)

        if (mark.icon) {
          if (mark.position === 'front') {
            content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
          } else {
            content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
          }
        } else if (mark.innerStyle) {
          content = <span style={mark.innerStyle}>{content}</span>
        } else if (mark.space) {
          content = <><span dangerouslySetInnerHTML={{__html: mark.space}}></span>{content}</>
        } else if (mark.point) {
          if (mark.position === 'front') {
            content = <>{mark.point}{content}</>
          } else {
            content = <>{content}{mark.point}</>
          }
        }
      }
      if (col.blur) {
        content = md5(content)
      }

      if (col.rowspan === 'true') {
        resProps.rowSpan = record['$$' + col.field]
      }
      
      if (!record.$disabled && (col.linkThdMenu || col.linkurl)) {
        content = (
          <div>
            <div className="link-menu" onDoubleClick={(e) => triggerLink(e, col, record)}></div>
            {content}
          </div>
        )
      }

      resProps.children = content
    } else if (col.type === 'number') {
      let content = ''
      try {
        content = parseFloat(record[col.field])
        if (isNaN(content)) {
          content = ''
        }
      } catch (e) {
        content = ''
      }

      if (content !== '') {
        if (col.round) {
          content = Math.round(content * col.round) / col.round
        }
        if (col.format === 'percent') {
          content = content * 100
        } else if (col.format === 'abs') {
          content = Math.abs(content)
        }
        if (col.round) {
          content = content.toFixed(col.decimal)
        }
  
        if (col.format === 'thdSeparator') {
          content = content.replace(/\d{1,3}(?=(\d{3})+(\.\d*)?$)/g, '$&,')
        }
  
        content = (col.prefix || '') + content + (col.postfix || '')
      }

      if (col.marks) {
        style = style ? {...style} : {}

        let mark = getMark(col.marks, record, style)

        if (mark.icon) {
          if (mark.position === 'front') {
            content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
          } else {
            content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
          }
        } else if (mark.innerStyle) {
          content = <span style={mark.innerStyle}>{content}</span>
        } else if (mark.space) {
          content = <><span dangerouslySetInnerHTML={{__html: mark.space}}></span>{content}</>
        } else if (mark.point) {
          if (mark.position === 'front') {
            content = <>{mark.point}{content}</>
          } else {
            content = <>{content}{mark.point}</>
          }
        }
      }

      if (col.blur) {
        content = md5(content)
      }

      if (col.rowspan === 'true') {
        resProps.rowSpan = record['$$' + col.field]
      }

      if (!record.$disabled && (col.linkThdMenu || col.linkurl)) {
        content = (
          <div>
            <div className="link-menu" onDoubleClick={(e) => triggerLink(e, col, record)}></div>
            {content}
          </div>
        )
      }

      resProps.children = content
    } else if (col.type === 'picture') {
      let photos = ''
      if (record[col.field]) {
        photos = `${record[col.field]}`
      }

      if (/^data:image/.test(photos)) {
        photos = [photos]
      } else {
        photos = photos.split(',').filter(Boolean)
      }

      let paddingTop = '100%'
      let scale = col.scale === 'true'

      if (PicRadio[col.lenWidRadio]) {
        paddingTop = PicRadio[col.lenWidRadio]
      }

      resProps.children = (
        <div>
          {photos.map((url, i) => (
            <Col key={i} span={col.span || 24}>
              <MkPicture lostTip={col.lostTip !== 'false'} style={{paddingTop, backgroundSize: col.backgroundSize || 'cover'}} scale={scale} url={url} urls={photos}/>
            </Col>
          ))}
        </div>
      )
    } else if (col.type === 'video') {
      let url = record[col.field] || ''

      resProps.children = (
        <div className="video-wrap">
          {url ? <Video card={col} value={url}/> : null}
        </div>
      )
    } else if (col.type === 'textarea') {
      let content = ''
      if (record[col.field] !== undefined) {
        content = `${record[col.field]}`
      }

      if (content) {
        content = (col.prefix || '') + content + (col.postfix || '')
      }

      if (col.blur) {
        content = md5(content)
      }

      resProps.children = (
        <div>
          {content ? <Paragraph copyable ellipsis={{ rows: 3, expandable: true }}>{content}</Paragraph> : null }
        </div>
      )
    } else if (col.type === 'link') {
      let content = ''
      let _href = record[col.field] || ''

      if (col.nameField) {
        content = record[col.nameField] || ''
      }

      if (!content && _href) {
        content = _href
      } else if (!_href) {
        content = ''
      }

      if (col.joint === 'true' && _href) {
        let _quary = `id=${record.$$uuid}&userid=${sessionStorage.getItem('UserID')}&appkey=${window.GLOB.appkey}&LoginUID=${sessionStorage.getItem('LoginUID')}`
        let _param = ''
        
        try {
          _param = window.btoa(_quary)
        } catch (e) {
          _param = window.btoa(window.encodeURIComponent(_quary))
        }
        
        if (/\?/ig.test(_href)) {
          _href += '&' + _param
        } else {
          _href += '?' + _param
        }
      }

      if (col.blur) {
        content = md5(content)
      }

      if (_href && /^https/.test(window.location.protocol)) { // https转换
        _href = _href.replace(/^http:/ig, 'https:')
      }

      resProps.children = (
        <div>
          {content && _href ? <a href={_href} target="_blank" rel="noopener noreferrer">{content}</a> : null }
        </div>
      )
    } else if (col.type === 'formula') {
      let content = col.formula

      if (col.eval === 'func') {
        try {
          // eslint-disable-next-line
          let func = new Function('data', col.formula)
          content = func([record])
        } catch (e) {
          console.warn(e)
          content = ''
        }
      } else {
        Object.keys(record).forEach(key => {
          let reg = new RegExp('@' + key + '@', 'ig')
          content = content.replace(reg, record[key])
        })
        if (col.eval !== 'false') {
          try {
            // eslint-disable-next-line
            content = eval(content)
          } catch (e) {
            console.info(content)
            console.warn(e)
            content = ''
          }
  
          if (col.round && typeof(content) === 'number') {
            content = Math.round(content * col.round) / col.round
            content = content.toFixed(col.decimal)
          }
        }
      }

      content = content === undefined ? '' : content

      if (col.eval === 'func') {
        content = <span dangerouslySetInnerHTML={{__html: content}}></span>
      } else if (content !== '') {
        content = `${col.prefix || ''}${content}${col.postfix || ''}`
        content = content.replace(/\n/ig, '<br/>').replace(/\s/ig, '&nbsp;')
        content = <span dangerouslySetInnerHTML={{__html: content}}></span>
      }

      if (col.marks) {
        style = style ? {...style} : {}

        let mark = getMark(col.marks, record, style)

        if (mark.icon) {
          if (mark.position === 'front') {
            content = <span><MkIcon style={mark.innerStyle} type={mark.icon} /> {content}</span>
          } else {
            content = <span>{content} <MkIcon style={mark.innerStyle} type={mark.icon} /></span>
          }
        } else if (mark.innerStyle) {
          content = <span style={mark.innerStyle}>{content}</span>
        } else if (mark.space) {
          content = <><span dangerouslySetInnerHTML={{__html: mark.space}}></span>{content}</>
        } else if (mark.point) {
          if (mark.position === 'front') {
            content = <>{mark.point}{content}</>
          } else {
            content = <>{content}{mark.point}</>
          }
        }
      }

      resProps.children = content
    } else if (col.type === 'custom') {
      style.padding = '0px'
      if (col.style) {
        style = {...style, ...col.style}
      }

      resProps.children = (
        <CardCellComponent data={record} cards={config} elements={col.elements}/>
      )
    }

    if (resProps.rowSpan === 0) return null
    return (<td {...resProps} className={className} style={style}/>)
  }
}

class NormalTable extends Component {
  static propTpyes = {
    statFValue: PropTypes.any,       // 合计字段数据
    MenuID: PropTypes.string,        // 菜单Id
    setting: PropTypes.object,       // 表格全局设置：tableType（表格是否可选、单选、多选）、columnfixed（列固定）、actionfixed（按钮固定）
    columns: PropTypes.array,        // 表格列
    lineMarks: PropTypes.array,      // 行标记
    fields: PropTypes.array,         // 组件字段集
    data: PropTypes.any,             // 表格数据
    total: PropTypes.any,            // 总数
    loading: PropTypes.bool,         // 表格加载中
    refreshdata: PropTypes.func,     // 表格中排序列、页码的变化时刷新
    chgSelectData: PropTypes.func,   // 数据切换
    autoMatic: PropTypes.any,
    allSearch: PropTypes.any,
    colsCtrls: PropTypes.any
  }

  state = {
    tableId: '',          // 表格ID
    selectedRowKeys: [],  // 表格中选中行
    pageIndex: 1,         // 初始页面索引
    pageSize: 10,         // 每页数据条数
    columns: null,        // 显示列
    activeIndex: null,    // 标记当前选中行
    rowspans: null,       // 行合并字段信息
    pickup: false,        // 收起未选择项
    orderfields: {},      // 排序id与field转换
    pageOptions: [],
    allColumns: null,
    reseting: false
  }

  UNSAFE_componentWillMount () {
    const { setting, columns, fields, colsCtrls } = this.props
    let radio = 5          // 虚化比例
    let _format = false    // 是否虚化处理
    let rowspans = []
    let orderfields = {}

    if (window.GLOB.dataFormat) {
      _format = true

      if (window.GLOB.memberLevel >= 30) {
        radio = 20
      } else if (window.GLOB.memberLevel >= 20) {
        radio = 10
      }
    }

    let _columns = []

    if (setting.tableMode !== 'fast') {
      let getColumns = (cols) => {
        return cols.map(item => {
          let cell = null
    
          if (item.type === 'colspan') {
            cell = { title: item.label, align: item.Align, $key: item.uuid }
            cell.children = getColumns(item.subcols)
          } else {
            if (item.rowspan === 'true') {
              rowspans.push(item.field)
            }
            if (item.type === 'index') {
              item.field = '$Index'
              item.type = 'text'
            } else if (_format && !Math.floor(Math.random() * radio)) {
              item.blur = true
            }
  
            if (item.field) {
              orderfields[item.uuid] = item.field
            } else if (item.sortField) {
              orderfields[item.uuid] = item.sortField
            }
  
            cell = {
              $key: item.uuid,
              align: item.Align,
              dataIndex: item.uuid,
              title: item.label,
              sorter: (item.field || item.sortField) && item.IsSort === 'true',
              width: item.Width || 120,
              onCell: record => ({
                record,
                col: item,
                config: item.type === 'custom' ? {setting, columns: fields} : null,
                triggerLink: this.triggerLink
              })
            }
          }
    
          return cell
        })
      }
      _columns = getColumns(columns)
    } else {
      let fields = []
      columns.forEach(item => {
        if (!item.field || fields.includes(item.field)) return
        fields.push(item.field)

        _columns.push({
          $key: item.uuid,
          align: item.Align,
          dataIndex: item.field,
          title: item.label,
          sorter: item.IsSort === 'true',
          width: item.Width || 120
        })
      })

      orderfields = null
    }

    if (rowspans.length === 0) {
      rowspans = null
    }

    let tableId = (() => {
      let uuid = []
      let _options = 'abcdefghigklmnopqrstuv'
      for (let i = 0; i < 19; i++) {
        uuid.push(_options.substr(Math.floor(Math.random() * 0x20), 1))
      }
      return uuid.join('')
    }) ()

    let size = (setting.pageSize || 10) + ''
    let pageOptions = ['10', '25', '50', '100', '500', '1000']

    if (!pageOptions.includes(size)) {
      pageOptions.push(size)
      pageOptions = pageOptions.sort((a, b) => a - b)
    }

    if (setting.maxPageSize) {
      pageOptions = pageOptions.filter(item => item <= setting.maxPageSize)
    }

    let allColumns = null
    if (colsCtrls) {
      rowspans = null
      allColumns = [..._columns]
      _columns = this.getCurColumns(_columns, this.props.allSearch)
    }

    this.setState({
      pageSize: setting.pageSize || 10,
      pageOptions,
      allColumns,
      columns: _columns,
      rowspans,
      tableId,
      orderfields
    })
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.props), fromJS(nextProps)) || !is(fromJS(this.state), fromJS(nextState))
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    const { allSearch } = this.props
    const { allColumns } = this.state

    if (allSearch && !is(fromJS(allSearch), fromJS(nextProps.allSearch))) {
      this.setState({
        reseting: true,
        columns: this.getCurColumns(allColumns, nextProps.allSearch)
      }, () => {
        this.setState({reseting: false})
      })
    }
  }

  componentDidMount () {
    const { autoMatic } = this.props

    MKEmitter.addListener('mkCheckAll', this.mkCheckAll)
    MKEmitter.addListener('resetTable', this.resetTable)

    if (autoMatic === true) {
      MKEmitter.addListener('autoQueryData', this.autoQueryData)
      MKEmitter.addListener('autoSelectData', this.autoSelectData)
    }

    MKEmitter.addListener('mkCheckTopLine', this.mkCheckTopLine)
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('mkCheckAll', this.mkCheckAll)
    MKEmitter.removeListener('resetTable', this.resetTable)
    MKEmitter.removeListener('autoQueryData', this.autoQueryData)
    MKEmitter.removeListener('autoSelectData', this.autoSelectData)
    MKEmitter.removeListener('mkCheckTopLine', this.mkCheckTopLine)
  }

  getCurColumns = (columns, allSearch) => {
    const { colsCtrls } = this.props

    let values = {}
    allSearch.forEach(item => {
      values[item.key] = item.value
    })
    let cols = null
    colsCtrls.some(item => {
      let originVal = item.field.map(f => values[f] || '').join('')
      let contrastVal = item.contrastValue
      let result = false

      if (item.match === '=') {
        result = originVal === contrastVal
      } else if (item.match === '!=') {
        result = originVal !== contrastVal
      } else if (item.match === 'regexp') {
        let reg = new RegExp(item.contrastValue, 'ig')
        result = reg.test(originVal)
      } else {
        originVal = isNaN(originVal) ? originVal : +originVal
        contrastVal = isNaN(contrastVal) ? contrastVal : +contrastVal
        if (item.match === '>') {
          result = originVal > contrastVal
        } else if (item.match === '<') {
          result = originVal < contrastVal
        }
      }

      if (!result) return false

      cols = item.cols

      return true
    })

    if (cols) {
      return columns.filter(col => cols.includes(col.$key))
    }

    return columns
  }

  autoSelectData = (id, index) => {
    if (id !== this.props.MenuID) return

    const { pageSize, pageIndex } = this.state

    let i = index - (pageIndex - 1) * pageSize - 1

    if (this.props.data[i]) {
      this.changeRow(this.props.data[i], i)
      MKEmitter.emit('autoTransSelectData', this.props.MenuID, this.props.data[i])
    } else {
      MKEmitter.emit('autoMaticOver', this.props.MenuID)
    }
  }

  autoQueryData = (id, index) => {
    if (id !== this.props.MenuID) return

    const { total } = this.props
    const { pageSize } = this.state

    if (index !== 1 && (!total || index > total)) {
      MKEmitter.emit('autoMaticOver', this.props.MenuID)
      return
    }

    console.clear()

    let pageIndex = Math.ceil(index / pageSize)

    this.setState({
      pageIndex: pageIndex,
      selectedRowKeys: [],
      activeIndex: null
    })

    this.props.refreshdata({pageIndex})
  }

  mkCheckTopLine = (menuId, id, type) => {
    const { MenuID, data, setting } = this.props

    if (MenuID !== menuId || !data || data.length === 0) return

    if (type === 'sign') {
      let index = ''
      let keys = []
      let items = []
      data.forEach((item, i) => {
        if (!item.$disabled && item.selected === 'true') {
          items.push(item)
          keys.push(i)
          index = i
        }
      })
  
      this.changedata(index)
      this.setState({ selectedRowKeys: keys, activeIndex: index })
      this.props.chgSelectData(items)

      if (setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', MenuID, items, data.length === keys.length)
      }
      return
    }

    let index = 0
    if (id) {
      index = data.findIndex(item => item.$$uuid === id)
      if (index === -1) {
        index = 0
      }
    }

    if (data[index].$disabled) return

    this.changedata(index)
    this.setState({ selectedRowKeys: [index], activeIndex: index })
    this.props.chgSelectData([data[index]])

    if (setting.$hasSyncModule) {
      MKEmitter.emit('syncBalconyData', MenuID, [data[index]], data.length === 1)
    }
  }

  mkCheckAll = (menuId, checked) => {
    const { MenuID, data } = this.props

    if (MenuID !== menuId) return

    if (checked) {
      this.setState({
        activeIndex: '',
        selectedRowKeys: data.map((item, index) => index)
      })

      this.props.chgSelectData(data)

      MKEmitter.emit('resetSelectLine', MenuID, '', '')
      MKEmitter.emit('syncBalconyData', MenuID, data, data.length > 0)
      if (data.length === 0) {
        message.warning('未获取到数据！')
      }
    } else {
      this.setState({
        activeIndex: '',
        selectedRowKeys: [],
      })

      this.props.chgSelectData([])
  
      MKEmitter.emit('resetSelectLine', MenuID, '', '')
      MKEmitter.emit('syncBalconyData', MenuID, [], false)
    }
  }

  // 字段透视
  triggerLink = (e, item, record) => {
    e.stopPropagation()

    let __param = {
      $searchkey: item.field,
      $searchval: record[item.field] || '',
      $BID: record.$$uuid
    }

    if (item.linkfields && item.linkfields.length > 0) {
      item.linkfields.forEach(field => {
        __param[field] = record[field] || ''
      })
    }

    if (item.linkThdMenu) {
      let tabmenu = item.linkThdMenu

      tabmenu.param = __param

      MKEmitter.emit('modifyTabs', tabmenu, true)
    } else if (item.linkurl) {
      let src = item.linkurl

      let con = '?'

      if (/\?/ig.test(src)) {
        con = '&'
      }

      if (item.linkfields && item.linkfields.length > 0) {
        item.linkfields.forEach(field => {
          if (field.toLowerCase() === 'id') return
          con += `${field}=${record[field] || ''}&`
        })
      }
      
      src = src + `${con}id=${record.$$uuid}&appkey=${window.GLOB.appkey}&userid=${sessionStorage.getItem('UserID')}&LoginUID=${sessionStorage.getItem('LoginUID') || ''}`

      window.open(src)
    }
  }

  onSelectChange = (selectedRowKeys, e) => {
    const { setting, MenuID, data } = this.props

    if (this.state.pickup) return

    let index = ''
    let _activeIndex = null
    if (selectedRowKeys.length > 0) {
      selectedRowKeys = selectedRowKeys.filter(key => !data[key].$disabled)
      index = selectedRowKeys.slice(-1)[0]
    }

    if (setting.tableType === 'checkbox' || setting.tableType === 'radio') {
      _activeIndex = index === '' ? null : index
    }

    this.changedata(index)

    this.setState({ selectedRowKeys, activeIndex: _activeIndex })

    let selects = data.filter((item, _index) => selectedRowKeys.includes(_index) && !item.$disabled)

    this.props.chgSelectData(selects)
    if (setting.$hasSyncModule) {
      MKEmitter.emit('syncBalconyData', MenuID, selects, data.length === selects.length)
    }
  }

  /**
   * @description 点击整行，触发切换， 判断是否可选，单选或多选，进行对应操作
   */
  changeRow = (record, index) => {
    const { setting, MenuID, data } = this.props

    if (!setting.tableType || this.state.pickup) return
    if (record.$disabled) return
    
    let newkeys = fromJS(this.state.selectedRowKeys).toJS()

    if (setting.tableType === 'radio') {
      newkeys = [index]
      this.changedata(index)
      this.setState({ selectedRowKeys: newkeys, activeIndex: index })
    } else {
      let _index = ''
      if (newkeys.includes(index)) {
        newkeys = newkeys.filter(item => item !== index)
        if (newkeys.length > 0) {
          _index = newkeys.slice(-1)[0]
        }
        this.changedata(_index)
      } else {
        _index = index
        newkeys.push(index)
        this.changedata(index)
      }

      this.setState({ selectedRowKeys: newkeys, activeIndex: _index !== '' ? _index : null })
    }

    let selects = this.props.data.filter((item, _index) => newkeys.includes(_index) && !item.$disabled)

    this.props.chgSelectData(selects)

    if (setting.$hasSyncModule) {
      MKEmitter.emit('syncBalconyData', MenuID, selects, data.length === selects.length)
    }
  }

  changeTable = (pagination, filters, sorter) => {
    const { orderfields } = this.state

    this.setState({
      pageIndex: pagination.current,
      pageSize: pagination.pageSize,
      selectedRowKeys: [],
      activeIndex: null,
      pickup: false
    })

    if (orderfields) {
      sorter.field = orderfields[sorter.field] || ''
    }

    this.props.refreshdata(pagination, filters, sorter)
  }

  changedata = (index) => {
    const { MenuID, data } = this.props

    let _id = ''
    let _data = ''

    if (data && data.length > 0 && index !== '') {
      _id = data[index].$$uuid || ''
      _data = data[index] || ''
    }

    if (_data && _data.$disabled) return

    MKEmitter.emit('resetSelectLine', MenuID, _id, _data)
  }

  resetTable = (id, type, Index) => {
    const { MenuID, setting, data } = this.props

    if (id !== MenuID) return

    if (type === 'delete') {
      let selectKeys = fromJS(this.state.selectedRowKeys).toJS()
      let activeKey = this.state.activeIndex

      let trans = activeKey === Index
      let _item = null
      let selects = []
      
      if (setting.tableType) {
        selectKeys = selectKeys.filter(key => key !== Index)
        selectKeys = selectKeys.map(key => key > Index ? key - 1 : key)

        selects = selectKeys.map(key => data[key]).filter(Boolean)

        activeKey = selectKeys.length ? selectKeys[selectKeys.length - 1] : null

        if (trans && selects.length) {
          _item = selects[selects.length - 1]
        }
      }

      this.setState({ selectedRowKeys: selectKeys, activeIndex: activeKey })

      this.props.chgSelectData(selects)

      if (trans) {
        MKEmitter.emit('resetSelectLine', MenuID, (_item ? _item.$$uuid : ''), _item)
      }

      if (setting.$hasSyncModule) {
        MKEmitter.emit('syncBalconyData', MenuID, selects, data.length === selects.length)
      }
    } else if (type === 'false') {
      this.setState({
        selectedRowKeys: [],
        activeIndex: null,
        pickup: false
      })
    } else if (type === 'repage') {
      this.setState({
        pageIndex: Index,
        selectedRowKeys: [],
        activeIndex: null,
        pickup: false
      })
    } else {
      this.setState({
        pageIndex: 1,
        selectedRowKeys: [],
        activeIndex: null,
        pickup: false
      })
    }
  }

  handleRowspan = (data) => {
    const { rowspans } = this.state

    if (!rowspans || data.length === 0) return data

    data = fromJS(data).toJS()
    data = data.reverse()
    data.forEach((item, index) => {
      if (index === 0) {
        rowspans.forEach(cell => {
          item['$' + cell] = 1
        })
      } else {
        let preItem = data[index - 1]
        rowspans.forEach((cell, i) => {
          if (i === 0) {
            if (preItem[cell] === item[cell]) {
              item['$' + cell] = preItem['$' + cell] + 1
            } else {
              item['$' + cell] = 1
            }
          } else {
            if (preItem[cell] === item[cell]) {
              item['$' + cell] = preItem['$' + cell] + 1
            } else {
              item['$' + cell] = 1
            }
            if (item['$' + cell] > item['$' + rowspans[i - 1]]) {
              item['$' + cell] = 1
            }
          }
        })
      }
    })
    data = data.reverse()
    data.forEach((item, index) => {
      if (index === 0) {
        rowspans.forEach(cell => {
          item['$$' + cell] = item['$' + cell]
        })
      } else {
        let preItem = data[index - 1]
        rowspans.forEach(cell => {
          if (preItem['$' + cell] > 1) {
            item['$$' + cell] = 0
          } else {
            item['$$' + cell] = item['$' + cell]
          }
        })
      }
    })
    return data
  }

  pickupChange = () => {
    this.setState({
      pickup: !this.state.pickup
    })
  }

  doubleClickLine = (record) => {
    const { setting } = this.props

    if (!setting.doubleClick) return
    if (record.$disabled) return

    MKEmitter.emit('triggerBtnId', setting.doubleClick, [record], 'linkbtn')
  }

  render() {
    const { setting, statFValue, lineMarks, data } = this.props
    const { selectedRowKeys, activeIndex, pickup, pageOptions, columns, reseting } = this.state

    if (reseting) return null

    // 设置表格选择属性：单选、多选、不可选
    let rowSelection = null
    if (setting.tableType) {
      rowSelection = {
        selectedRowKeys,
        type: (setting.tableType === 'radio') ? 'radio' : 'checkbox',
        onChange: this.onSelectChange
      }
    }

    let components = {
      body: {
        row: BodyRow
      }
    }

    if (setting.tableMode !== 'fast') {
      components.body.cell = BodyCell
    }

    // 数据收起时，过滤已选数据
    let _data = data || []

    if (pickup) {
      _data = _data.filter((item, index) => selectedRowKeys.includes(index))
    }

    _data = this.handleRowspan(_data)
    
    let _pagination = false
    if (setting.laypage !== 'false' && setting.laypage !== false) {
      _pagination = {
        current: this.state.pageIndex,
        pageSize: this.state.pageSize,
        pageSizeOptions: pageOptions,
        showSizeChanger: true,
        total: this.props.total || 0,
        showTotal: (total, range) => `${range[0]}-${range[1]} 共 ${total} 条`
      }
    }

    let _footer = ''

    if (statFValue && statFValue.length > 0) {
      _footer = statFValue.map(f => `${f.label}(合计)：${f.value}`).join('；')
    }

    let height = setting.height || false
    if (height && height <= 100) {
      height = height + 'vh'
    }
    let loading = this.props.loading
    if (setting.mask === 'hidden') {
      loading = false
    }

    let fixed = ''
    if (setting.colfixed && setting.colfixed.length) {
      if (setting.colfixed.includes('first')) {
        fixed = 'mk-fixed-first-col'
      }
      if (setting.colfixed.includes('last')) {
        fixed += ' mk-fixed-last-col'
      }
    }

    let style = {
      '--mk-table-border-color': setting.borderColor || '#e8e8e8',
      '--mk-table-color': setting.color || 'rgba(0, 0, 0, 0.65)',
      '--mk-table-font-size': setting.fontSize || '14px',
      '--mk-table-font-weight': setting.fontWeight || 'normal'
    }

    return (
      <div className={`normal-custom-table ${setting.tableHeader || ''} ${height ? 'fixed-height' : ''} ${setting.mode || ''} table-vertical-${setting.vertical || 'middle'} table-col-${columns.length} ${fixed}`} style={style}>
        {(setting.tableType === 'radio' || setting.tableType === 'checkbox') && data && data.length > 0 ?
          <Switch title="收起" className="main-pickup" checkedChildren="开" unCheckedChildren="关" checked={pickup} onChange={this.pickupChange} /> : null
        }
        <Table
          components={components}
          size={setting.size || 'middle'}
          bordered={setting.bordered !== 'false'}
          rowSelection={rowSelection}
          columns={columns}
          dataSource={_data}
          loading={loading}
          scroll={{ x: '100%', y: height }}
          onRow={(record, index) => {
            return {
              lineMarks: setting.tableMode !== 'fast' ? lineMarks : null,
              data: record,
              title: setting.tipField ? record[setting.tipField] : '',
              className: index === activeIndex ? ' mk-row-active ' : '',
              onClick: () => {this.changeRow(record, index)},
              onDoubleClick: () => {this.doubleClickLine(record)}
            }
          }}
          onChange={this.changeTable}
          pagination={_pagination}
        />
        {_footer ? <div className={'normal-table-footer ' + (_pagination ? 'pagination' : '')}>{_footer}</div> : null}
      </div>
    )
  }
}

export default NormalTable